@@ -2,19 +2,21 @@
 #define COMBO_RESULTS_H
 
 #include <Rcpp.h>
-#include <ConstraintsUtils.h>
+#include "ConstraintsUtils.h"
 
 template <typename typeMatrix, typename typeVector>
 void ComboGenRes(int n, int r, std::vector<typeVector> &v, bool repetition,
-                  int nRows, int count, std::vector<int> z,
-                  typeMatrix combinationMatrix, funcPtr<typeVector> myFun) {
+                  int nRows, int count, std::vector<int> &z,
+                  typeMatrix &combinationMatrix, funcPtr<typeVector> myFun) {
     
-    int r1 = r - 1, r2 = r - 2, numIter;
+    const int r1 = r - 1;
+    const int r2 = r - 2;
+    int numIter;
     std::vector<typeVector> vPass(r);
     unsigned long int uR = r;
     
     if (repetition) {
-        int lastElement = n - 1;
+        const int lastElement = n - 1;
         
         while (count < nRows) {
             numIter = n - z[r1];
@@ -42,7 +44,7 @@ void ComboGenRes(int n, int r, std::vector<typeVector> &v, bool repetition,
             }
         }
     } else {
-        int nMinusR = n - r;
+        const int nMinusR = n - r;
         
         while (count < nRows) {
             numIter = n - z[r1];
@@ -75,7 +77,7 @@ void ComboGenRes(int n, int r, std::vector<typeVector> &v, bool repetition,
 template <typename typeMatrix, typename typeVector>
 void MultisetComboResult(int n, int r, std::vector<typeVector> &v, std::vector<int> &Reps,
                          std::vector<int> &freqs, int nRows, int count,
-                         std::vector<int> z, typeMatrix combinationMatrix,
+                         std::vector<int> &z, typeMatrix &combinationMatrix,
                          funcPtr<typeVector> myFun) {
     
     std::vector<int> zIndex(n), zGroup(r);
